SystemOrganization addCategory: #'WebStore-Core'!
SystemOrganization addCategory: #'WebStore-TestSite'!
SystemOrganization addCategory: #'WebStore-Components'!
SystemOrganization addCategory: #'WebStore-Admin'!
SystemOrganization addCategory: #'WebStore-Tests'!
SystemOrganization addCategory: #'WebStore-Util'!

WAComponent subclass: #TestSite
	instanceVariableNames: 'store products'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-TestSite'!

!TestSite class methodsFor: 'as yet unclassified' stamp: 'srh 12/28/2008 12:53'!
canBeRoot
	^true! !

!TestSite methodsFor: 'as yet unclassified' stamp: 'srh 12/28/2008 13:34'!
children
	^Array with: products.! !

!TestSite methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 17:42'!
initialize
	| rs |
	super initialize.
	rs := ReferenceStream fileNamed: 'demo_store'.
	store := rs next.
	rs close.
	products := WSProductList store: store.! !

!TestSite methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 18:35'!
renderContentOn: html
	html heading: 'Emma''s Store'.
	html div id: 'nav';with: [
		html unorderedList with: [
			html listItem: [ html anchor callback: []; with: 'Home'].
			html listItem: [ html anchor callback: []; with: 'A link']
		]].
	html div id: 'content';with:products! !

WAComponent subclass: #WSField
	instanceVariableNames: 'value selector receiver tag label rules'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Util'!

!WSField class methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 18:57'!
largeText
	^self basicNew
		tagType:  #textArea;
		initialize;
		yourself.! !

!WSField class methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 18:15'!
money
	^WSMoneyField new.! !

!WSField class methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 18:13'!
text
	^self basicNew
		tagType:  #textInput;
		initialize;
		yourself.! !

!WSField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:05'!
addRule: aBlock withDescription: desc.
	rules add: (WSRule rule: aBlock message: desc)! !

!WSField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:07'!
beRequired
	"Nil is allowed, because the only time it'll be nil is when the form is first rendered"
	self addRule: [ :v| v isNil or: [v notEmpty]] withDescription: label, ' is a required field'.! !

!WSField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 17:25'!
initialize
	super initialize.
	rules := OrderedCollection new.! !

!WSField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 17:05'!
label: aString
	label := aString! !

!WSField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:06'!
messages
	^rules select: [ :r | (r testRule: value) not] thenCollect: [ :r | r message ] ! !

!WSField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:14'!
on: aSelector of: aReceiver
	selector := aSelector.
	receiver := aReceiver.
	self setValue! !

!WSField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 17:11'!
renderContentOn: html
	html div class: (self valid ifFalse: ['error']); with: [
		| id |
		id := html nextId.
		html label for: id; with: label.
		(html perform: tag) 
			id: id;
			callback: [ :v| value := v];
			value: value
		]! !

!WSField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:08'!
save
	receiver perform: selector asMutator with: value! !

!WSField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:14'!
setValue
	value := receiver perform: selector! !

!WSField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 17:01'!
tagType: symbol
	tag := symbol! !

!WSField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:06'!
valid
	^rules allSatisfy: [ :r | r  testRule: value ]! !

WSField subclass: #WSMoneyField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Util'!

!WSMoneyField class methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 18:18'!
new
	^self basicNew
		tagType: #textInput;
		initialize;
		yourself.! !

!WSMoneyField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:18'!
save
	receiver perform: selector asMutator with: ((value asNumber) * 100) asInteger! !

!WSMoneyField methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:22'!
setValue
	value := receiver perform: selector.
	value ifNotNil: [ value := (value / 100) asFloat printString]! !

WAComponent subclass: #WSManageProducts
	instanceVariableNames: 'store'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Admin'!

!WSManageProducts class methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 12:02'!
store: aStore
	^self basicNew
		store: aStore;
		initialize;
		yourself! !

!WSManageProducts methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 20:01'!
addProduct
	| p |
	p := self call: WSProductEditor new.
	store addProduct: p! !

!WSManageProducts methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 14:09'!
deleteProduct: p
	(self confirm: 'Really delete ',p name, '?') ifTrue: [
		store deleteProduct: p	
	]! !

!WSManageProducts methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 20:01'!
editProduct: aProduct 
	self call: (WSProductEditor product: aProduct)! !

!WSManageProducts methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 18:48'!
initialize
	super initialize! !

!WSManageProducts methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 12:11'!
renderActionsOn: html
	html div: [
		html anchor callback: [ self addProduct]; with: 'Add Product'].! !

!WSManageProducts methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 12:06'!
renderContentOn: html

	self renderStatsOn: html.
	self renderActionsOn: html.
	self renderProductListOn: html.

	html anchor callback: [ self answer ]; with: 'back'.! !

!WSManageProducts methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 13:50'!
renderProductListOn: html
	html table: [
		html tableRow: [
			html tableHeading.
			html tableHeading: 'Name'.
			html tableHeading: 'Price'.
			html tableHeading: 'Actions'	].
		store products do: [ :p| html tableRow: [
			html tableData.		
			html tableData: p name.
			html tableData: p priceInDollars printString.
			html tableData: [ 
				html anchor callback: [ self deleteProduct: p ]; with: 'Delete'.
				html space.
				html anchor callback: [self editProduct: p]; with: 'Edit'.
		]]]]! !

!WSManageProducts methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 12:08'!
renderStatsOn: html
	html div: [
		html paragraph: store products size printString, ' products'.
	]! !

!WSManageProducts methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 12:03'!
store: aStore
	store := aStore! !

WAComponent subclass: #WSProductDetails
	instanceVariableNames: 'product'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Components'!

!WSProductDetails class methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 17:54'!
for: aProduct
	^self basicNew
		product: aProduct;
		initialize;
		yourself.! !

!WSProductDetails methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 18:47'!
initialize
	super initialize.! !

!WSProductDetails methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 17:55'!
product: aProduct
	product := aProduct.! !

!WSProductDetails methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 18:34'!
renderContentOn: html
	html heading level: 2; with: product name.
	html paragraph with: '$', (product price / 100) asInteger printString.
	html paragraph with: product description.
	html anchor callback: [ self answer ]; with: 'Back'.! !

WAComponent subclass: #WSProductEditor
	instanceVariableNames: 'product fields'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Admin'!

!WSProductEditor class methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 18:55'!
new
	^self basicNew
		product: (WSProduct new);
		initialize;
		yourself.! !

!WSProductEditor class methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 15:49'!
product: aProduct
	^self basicNew
		product: aProduct;
		initialize;
		yourself.! !

!WSProductEditor methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 15:39'!
children
	^fields! !

!WSProductEditor methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 16:47'!
initialize
	super initialize.
	self setupFields.
! !

!WSProductEditor methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 15:48'!
product: aProduct
	product := aProduct! !

!WSProductEditor methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 15:46'!
renderContentOn: html
	html form: [
		fields do: [:f | html render: f].
		html div: [
			html cancelButton.
			html submitButton callback: [ self saveAndAnswer]
	]]! !

!WSProductEditor methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 15:48'!
saveAndAnswer
	self valid ifTrue: [ 
		fields do: [ :f| f save].
		self answer: product]! !

!WSProductEditor methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:10'!
setupFields
	fields := OrderedCollection new.
	
	fields add: (WSField text 
		label: 'Name';
		on: #name of: product;
		beRequired).
	fields add: (WSField money
		label: 'Price';
		on: #price of: product;
		beRequired).
	fields add: (WSField largeText
		label: 'Description';
		on: #description of: product).! !

!WSProductEditor methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 15:45'!
valid
	^fields allSatisfy: [:f| f valid]! !

WAComponent subclass: #WSProductList
	instanceVariableNames: 'store'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Components'!

!WSProductList class methodsFor: 'as yet unclassified' stamp: 'srh 12/28/2008 13:39'!
store: aStore
	^self basicNew 
		store: aStore;
		initialize;
		yourself.! !

!WSProductList methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 18:30'!
renderContentOn: html
	store products do: [ :p| (WSProductPreview for: p on: self) renderContentOn: html]! !

!WSProductList methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 18:28'!
store: aStore
	store := aStore.! !

WAComponent subclass: #WSStoreAdmin
	instanceVariableNames: 'store'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Admin'!

!WSStoreAdmin class methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 11:51'!
canBeRoot
	^true! !

!WSStoreAdmin methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 12:01'!
initialize
	|r|
	super initialize.
	r := ReferenceStream fileNamed: 'demo_store'.
	store := r next.
	r close.! !

!WSStoreAdmin methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 11:54'!
renderContentOn: html
	html heading level: 2;with: [
		html anchor 
			callback: [ self call: (WSManageProducts store: store)]; 
			with: 'Manage Products'	]! !

Object subclass: #WSImage
	instanceVariableNames: 'fullsize thumbnail medium'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Components'!

!WSImage class methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 11:48'!
fromFile: aFile
	! !

!WSImage methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 11:41'!
fullsize
	^fullsize! !

!WSImage methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 11:41'!
medium
	^medium! !

!WSImage methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 11:41'!
thumbnail
	^thumbnail! !

Object subclass: #WSProduct
	instanceVariableNames: 'name description price images'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Core'!

!WSProduct methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 07:29'!
description
	^description! !

!WSProduct methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 07:30'!
description: aDescription
	description := aDescription.! !

!WSProduct methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 21:22'!
images
	^images! !

!WSProduct methodsFor: 'as yet unclassified' stamp: 'srh 12/28/2008 13:53'!
name
	^name.! !

!WSProduct methodsFor: 'as yet unclassified' stamp: 'srh 12/28/2008 13:53'!
name: aName
	name := aName.! !

!WSProduct methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 07:32'!
price
	^price! !

!WSProduct methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 07:32'!
price: priceInCents
	price := priceInCents! !

!WSProduct methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:20'!
priceInDollars
	^(price / 100) asFloat! !

Object subclass: #WSProductPreview
	instanceVariableNames: 'product parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Components'!

!WSProductPreview class methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 18:27'!
for: aProduct on: parent
	^self basicNew 
		product: aProduct; 
		parent: parent;
		initialize;
		yourself.! !

!WSProductPreview methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 17:48'!
initialize
	super initialize.! !

!WSProductPreview methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 18:26'!
parent: aComponent
	parent := aComponent! !

!WSProductPreview methodsFor: 'as yet unclassified' stamp: 'srh 12/29/2008 17:48'!
product: aProduct
	product := aProduct.! !

!WSProductPreview methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 14:04'!
renderContentOn: html
	html paragraph: '$', product priceInDollars printString.
	html div: [html heading level: 3; with: [
			html anchor callback: [ parent call: (WSProductDetails for: product) ]; with: product name]]! !

Object subclass: #WSRule
	instanceVariableNames: 'rule message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Util'!

!WSRule class methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:01'!
rule: aBlock message: aMessage
	^self basicNew
		rule: aBlock;
		message: aMessage;
		initialize;
		yourself.! !

!WSRule methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:02'!
message
	^message! !

!WSRule methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:02'!
message: aString
	message := aString! !

!WSRule methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:02'!
rule
	^rule! !

!WSRule methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:01'!
rule: aBlock
	rule :=  aBlock! !

!WSRule methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 19:04'!
testRule: value
	^rule value: value! !

Object subclass: #WSStore
	instanceVariableNames: 'products'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Core'!

!WSStore methodsFor: 'as yet unclassified' stamp: 'srh 12/28/2008 13:48'!
addProduct: aProduct
	products add: aProduct.! !

!WSStore methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 14:45'!
deleteProduct: aProduct
	products remove: aProduct! !

!WSStore methodsFor: 'as yet unclassified' stamp: 'srh 12/28/2008 13:44'!
initialize
	super initialize.
	products := OrderedCollection new.! !

!WSStore methodsFor: 'as yet unclassified' stamp: 'srh 12/28/2008 13:49'!
products
	^OrderedCollection new addAll: products.! !

TestCase subclass: #WSProductTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Tests'!

!WSProductTest methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 14:03'!
testPriceInDollars
	self assert: 30.45 equals: (WSProduct new price: 3045;priceInDollars)! !

TestCase subclass: #WSStoreTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebStore-Tests'!

!WSStoreTest methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 14:42'!
testAddProduct
	| s p |
	s := WSStore new.
	p := WSProduct new.
	s addProduct: p.
	self assert: (s products includes: p).
	! !

!WSStoreTest methodsFor: 'as yet unclassified' stamp: 'srh 12/30/2008 14:46'!
testDeleteProduct
	| s p |
	s := WSStore new.
	p := WSProduct new.
	s addProduct: p.
	s deleteProduct: p.
	self assert: (s products includes: p) not	! !
